
(use-package slime
  :commands (slime-eval-buffer slime-eval-defun)
  :bind
  (:map slime-mode-map
        ("M-n" . next-line)
        ("M-p" . previous-line)
        ("SPC" . self-insert-command)
        ("<f1> <f1>" . slime-documentation))
  :config
  (slime-setup)
  (setq inferior-lisp-program
        ;; "/usr/bin/ecl"
        "/home/anon/bin/sbcl"
        ;; "/usr/bin/sbcl"
        slime-net-coding-system 'utf-8-unix))

(use-package slime-tramp
  :load-path "/home/anon/.emacs.d/elpa/slime-20210202.1426/contrib/"
  :config
  (add-to-list 'slime-filename-translations
               (slime-create-filename-translator
                :machine-instance "tilde"
                :remote-host "tilde.team"
                :username "contrapunctus")))

(defun set-cl-repl-keys ()
  (if (or (cp-buffer-name-match-p "^\*inferior-lisp\*\\(<[0-9]*>\\)?")
          (eq major-mode 'lisp-mode))
      (progn
        (local-set-key (kbd "TAB") 'completion-at-point)
        (local-set-key (kbd "M-q") 'sp-indent-defun))))

(add-hook 'slime-connected-hook 'set-cl-repl-keys)
(add-hook 'comint-mode-hook     'set-cl-repl-keys)
(add-hook 'lisp-mode-hook       'set-cl-repl-keys)

(setq scheme-program-name "csi -:c")
(setq comint-prompt-read-only t)

(use-package geiser
  :mode ("\\.scm\\'" . geiser-mode)
  :commands (run-chicken run-guile geiser-mode))
;; (with-eval-after-load 'geiser-mode
;;   (setq geiser-mode-smart-tab-p t)
;;   (define-key geiser-mode-map (kbd "C-.") nil)
;;   ;; (cp-set-keys
;;   ;;  :unset t
;;   ;;  :keymap geiser-mode-map
;;   ;;  :bindings
;;   ;;  `((,(kbd "C-."))))
;;   )

(use-package scheme-mode
  :mode ("\\.scm\\'" . scheme-mode)
  :interpreter "csi")

(defun contrapunctus-lisp-copy (arg)
  "Run `whole-line-or-region-copy-region-as-kill' if region is
active, else `sp-copy-sexp'."
  (interactive "P")
  (if (region-active-p)
      (whole-line-or-region-copy-region-as-kill arg)
    (sp-copy-sexp arg)))

(use-package projectile
  :hook
  (emacs-lisp-mode . projectile-mode))

(defun colorize-compilation-buffer ()
  (ansi-color-apply-on-region compilation-filter-start
			      (point)))

(add-hook 'compilation-filter-hook 'colorize-compilation-buffer)

;; Stopped calling `exec-path-from-shell-initialize' on init, but
;; won't this keep running it excessively?
;; `exec-path-from-shell-copy-envs' (which it calls internally)
;; doesn't seem idempotent, either.
(use-package exec-path-from-shell
  :hook
  (compilation-mode . exec-path-from-shell-initialize)
  (shell-mode-hook . exec-path-from-shell-initialize)
  (minibuffer-setup-hook . exec-path-from-shell-initialize))

(provide 'cp-lisp)
